/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

#include <qpid/messaging/Connection.h>
#include <qpid/messaging/Message.h>
#include <qpid/messaging/Receiver.h>
#include <qpid/messaging/Sender.h>
#include <qpid/messaging/Session.h>

#include <iostream>
#include <stdlib.h>
#include <ctime>

#include "OptionParser.h"

using namespace qpid::messaging;
using namespace qpid::types;

struct Options : OptionParser
{
    std::string url;
    std::string address;
    std::string connectionOptions;
    int timeout;
    int delay;
    int variation;
    int output_msg_print;
    bool forever;
    int count;
    int capacity;
    int contentSize;

    Options()
        : OptionParser("Usage: tx_test [OPTIONS] ADDRESS", "sends and receives messages to/from the specified address in transactions"),
          url("127.0.0.1"),
          timeout(0),
	  delay(0),
	  variation(0),
	  output_msg_print(0),
          forever(false),
          count(1),
	  capacity(0),
	  contentSize(0)
    {
        add("broker,b", url, "url of broker to connect to");
        add("timeout,t", timeout, "timeout in seconds to wait before exiting");
	add("delay,d", delay, "delay / sleep after receiving each message (in ms)");
	add("variation,v", variation, "variation in receiving message (in ms) - next message will be received after d+-rand(v) milliseconds");
	add("output_msg,o", output_msg_print, "output each Nth message sent (0=disabled)");
        add("forever,f", forever, "ignore timeout and wait forever");
        add("connection-options", connectionOptions, "connection options string in the form {name1=value1, name2=value2}");
        add("count,c", count, "number of messages to read before exiting");
	add("capacity,C", capacity, "capacity of the receiver");
	add("content-size,s", contentSize, "size of message content / body");
    }

    Duration getTimeout()
    {
        if (forever) return Duration::FOREVER;
        else return timeout*Duration::SECOND;
    }

    int getCount()
    {
        return count;
    }
    
    bool checkAddress()
    {
        if (getArguments().empty()) {
            error("Address is required");
            return false;
        } else {
            address = getArguments()[0];
            return true;
        }
    }
};

int main(int argc, char** argv)
{
    srandom(time(NULL));
    int _return_value=0;
    Connection connection;
    try {
    Options options;
    if (options.parse(argc, argv) && options.checkAddress()) {
	int count = options.getCount();
	connection = Connection(options.url, options.connectionOptions);
	connection.open();
	Session session = connection.createSession();
	Receiver receiver = session.createReceiver(options.address);
	Sender sender = session.createSender(options.address);
	Message message;
	Message msg;
	if (options.contentSize > 0) {
	    std::string str=""; for (int i=0; i<options.contentSize; i++) str+="A";
            message.setContent(str);
            message.setContentType("text/plain");
	    message.setDurable(true);
        }

	int queueSize=0;
	for (int i=0; i<count; i++) {
	    if ((queueSize>0) && (rand()%1000>0)) {
		msg = receiver.fetch(Duration::SECOND * 1); queueSize--;
	    }
	    else {
		sender.send(message); queueSize++;
	    }
	    if (i%100 == 0)
		session.acknowledge();
	}
	session.close();
	connection.close();
    }
    } catch(const std::exception& error) {
        std::cout << error.what() << std::endl;
        connection.close();
        _return_value++;
    }
    return _return_value;
}
